@@ -1,6 +1,4 @@
 package Text::Xslate::PP::Type::Array;
-use Carp;
-carp __PACKAGE__, " has been deprecated";
 1;
 __END__
 
